@@ -24,6 +24,46 @@ class PermissionsTestCase(APITestCase):
 
         self.circle = Circle.objects.create(name='Test', status=status_choice, owner=owner)
 
+    def _get_circle_member_request_json(self, user, circle=None):
+        res = self._get_request_json(user=user.urlid)
+
+        if circle is not None:
+            res.update({'circle': {'@id': circle.urlid}})
+
+        return res
+
+    def _get_request_json(self, **kwargs):
+        res = {
+            '@context': {
+                '@vocab': "http://happy-dev.fr/owl/#",
+                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
+                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
+                'ldp': "http://www.w3.org/ns/ldp#",
+                'foaf': "http://xmlns.com/foaf/0.1/",
+                'name': "rdfs:label",
+                'acl': "http://www.w3.org/ns/auth/acl#",
+                'permissions': "acl:accessControl",
+                'mode': "acl:mode",
+                'inbox': "http://happy-dev.fr/owl/#inbox",
+                'object': "http://happy-dev.fr/owl/#object",
+                'author': "http://happy-dev.fr/owl/#author",
+                'account': "http://happy-dev.fr/owl/#account",
+                'jabberID': "foaf:jabberID",
+                'picture': "foaf:depiction",
+                'firstName': "http://happy-dev.fr/owl/#first_name",
+                'lastName': "http://happy-dev.fr/owl/#last_name",
+                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
+            }
+        }
+
+        for kwarg in kwargs:
+            if isinstance(kwargs[kwarg], str):
+                res.update({kwarg: {'@id': kwargs[kwarg]}})
+            else:
+                res.update({kwarg: kwargs[kwarg]})
+
+        return res
+
     def _get_random_user(self):
         return User.objects.create(email='{}@test.co.uk'.format(str(uuid.uuid4())), first_name='Test', last_name='Test',
                                    username=str(uuid.uuid4()))
@@ -114,30 +154,7 @@ class PermissionsTestCase(APITestCase):
 
         self.setUpCircle('Private', another_user)
 
-        payload = {
-            'user': {'@id': self.user.urlid},
-            'circle': {'@id': self.circle.urlid},
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        payload = self._get_request_json(circle=self.circle.urlid, user=another_user.urlid)
 
         response = self.client.post('/circle-members/', json.dumps(payload), content_type='application/ld+json')
         self.assertEqual(response.status_code, 403)
@@ -148,30 +165,7 @@ class PermissionsTestCase(APITestCase):
 
         self.setUpCircle('Private', another_user)
 
-        payload = {
-            'user': {'@id': self.user.urlid},
-            'circle': {'@id': self.circle.urlid},
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        payload = self._get_circle_member_request_json(self.user, self.circle)
 
         response = self.client.post('/circles/1/members/', json.dumps(payload), content_type='application/ld+json')
         self.assertEqual(response.status_code, 403)
@@ -190,29 +184,7 @@ class PermissionsTestCase(APITestCase):
         self.setUpCircle('Private', another_user)
         CircleMember.objects.create(circle=self.circle, user=self.user, is_admin=False)
 
-        payload = {
-            'user': {'@id': self.user.urlid},
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        payload = self._get_circle_member_request_json(self.user)
 
         response = self.client.post('/circles/1/members/', json.dumps(payload), content_type='application/ld+json')
         self.assertEqual(response.status_code, 201)
@@ -254,30 +226,7 @@ class PermissionsTestCase(APITestCase):
 
         another_user = self._get_random_user()
 
-        body = {
-            'user': {'@id': another_user.urlid},
-            'circle': {'@id': self.circle.urlid},
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        body = self._get_circle_member_request_json(another_user, self.circle)
 
         response = self.client.post('/circle-members/', data=json.dumps(body),
                                     content_type='application/ld+json')
@@ -292,30 +241,7 @@ class PermissionsTestCase(APITestCase):
 
         another_user = self._get_random_user()
 
-        body = {
-            'user': {'@id': another_user.urlid},
-            'circle': {'@id': self.circle.urlid},
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        body = self._get_circle_member_request_json(another_user)
 
         response = self.client.post('/circles/1/members/', data=json.dumps(body),
                                     content_type='application/ld+json')
@@ -324,14 +250,24 @@ class PermissionsTestCase(APITestCase):
         self.assertEqual(len(circle.members.all()), 2)
 
         another_user = self._get_random_user()
+        body = self._get_circle_member_request_json(another_user, self.circle)
 
-        body.pop('user')
         response = self.client.post('/users/{}/circles/'.format(another_user.pk), data=json.dumps(body),
                                     content_type='application/ld+json')
         self.assertEqual(response.status_code, 201)
         circle = Circle.objects.get(pk=self.circle.pk)
         self.assertEqual(len(circle.members.all()), 3)
 
+    def test_post_circle_member_no_circle_passed(self):
+        self.setUpLoggedInUser()
+        self.setUpCircle('Private', self.user)
+
+        another_user = self._get_random_user()
+        body = self._get_circle_member_request_json(another_user)
+        response = self.client.post('/users/{}/members/'.format(another_user.pk), data=json.dumps(body),
+                                    content_type='application/ld+json')
+        self.assertEqual(response.status_code, 404)
+
     # removing a CircleMember - I am an admin
     def test_delete_circle_member_admin(self):
         self.setUpLoggedInUser()
@@ -399,29 +335,7 @@ class PermissionsTestCase(APITestCase):
         another_user = self._get_random_user()
         CircleMember.objects.create(circle=self.circle, user=another_user, is_admin=False)
 
-        body = {
-            'is_admin': True,
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        body = self._get_request_json(is_admin=True)
 
         response = self.client.patch('/circle-members/2/', json.dumps(body), content_type='application/ld+json')
 
@@ -463,29 +377,7 @@ class PermissionsTestCase(APITestCase):
         another_user = self._get_random_user()
         circle = Circle.objects.create(name='Test 2', status='Private', owner=another_user)
 
-        payload = {
-            'circle': {'@id': circle.urlid},
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        payload = self._get_request_json(circle=circle.urlid)
 
         response = self.client.patch('/circle-members/1/', json.dumps(payload), content_type='application/ld+json')
         self.assertEqual(response.status_code, 403)
@@ -503,30 +395,7 @@ class PermissionsTestCase(APITestCase):
         another_user = self._get_random_user()
         circle = Circle.objects.create(name='Test 2', status='Private', owner=another_user)
 
-        payload = {
-            'circle': {'@id': circle.urlid},
-            'is_admin': True,
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        payload = self._get_request_json(circle=circle.urlid, is_admin=True)
 
         response = self.client.patch('/circle-members/1/', data=json.dumps(payload),
                                      content_type='application/ld+json')
@@ -540,31 +409,7 @@ class PermissionsTestCase(APITestCase):
         another_user = self._get_random_user()
         self.setUpCircle('Private', another_user)
 
-        payload = {
-            'circle': {'@id': self.circle.urlid},
-            'user': {'@id': self.user.urlid},
-            'is_admin': True,
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        payload = self._get_request_json(circle=self.circle.urlid, user=self.user.urlid, is_admin=True)
 
         response = self.client.post('/circles/1/members/', data=json.dumps(payload), content_type='application/ld+json')
         self.assertEqual(response.status_code, 403)
@@ -576,31 +421,7 @@ class PermissionsTestCase(APITestCase):
 
         CircleMember.objects.create(user=self.user, circle=self.circle, is_admin=False)
 
-        payload = {
-            'circle': {'@id': self.circle.urlid},
-            'user': {'@id': self.user.urlid},
-            'is_admin': True,
-            '@context': {
-                '@vocab': "http://happy-dev.fr/owl/#",
-                'rdf': "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                'rdfs': "http://www.w3.org/2000/01/rdf-schema#",
-                'ldp': "http://www.w3.org/ns/ldp#",
-                'foaf': "http://xmlns.com/foaf/0.1/",
-                'name': "rdfs:label",
-                'acl': "http://www.w3.org/ns/auth/acl#",
-                'permissions': "acl:accessControl",
-                'mode': "acl:mode",
-                'inbox': "http://happy-dev.fr/owl/#inbox",
-                'object': "http://happy-dev.fr/owl/#object",
-                'author': "http://happy-dev.fr/owl/#author",
-                'account': "http://happy-dev.fr/owl/#account",
-                'jabberID': "foaf:jabberID",
-                'picture': "foaf:depiction",
-                'firstName': "http://happy-dev.fr/owl/#first_name",
-                'lastName': "http://happy-dev.fr/owl/#last_name",
-                'isAdmin': "http://happy-dev.fr/owl/#is_admin"
-            }
-        }
+        payload = self._get_request_json(circle=self.circle.urlid, user=self.user.urlid, is_admin=True)
 
         self.client.post('/circles/1/members/', data=json.dumps(payload), content_type='application/ld+json')
         self.assertFalse(CircleMember.objects.filter(user=self.user, circle=self.circle, is_admin=True).exists())
@@ -609,34 +430,14 @@ class PermissionsTestCase(APITestCase):
         self.setUpLoggedInUser()
         self.setUpCircle('Public', self.user)
 
-        payload = {
-            "description": "Test Circle",
-            "name": "Test Circle 1",
-            "owner": {"@id": "https://distant.com/users/1/"},
-            "@id": self.circle.urlid,
-            "@context": {"@vocab": "http://happy-dev.fr/owl/#",
-                         "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                         "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
-                         "ldp": "http://www.w3.org/ns/ldp#",
-                         "foaf": "http://xmlns.com/foaf/0.1/",
-                         "name": "rdfs:label",
-                         "acl": "http://www.w3.org/ns/auth/acl#",
-                         "permissions": "acl:accessControl",
-                         "mode": "acl:mode",
-                         "geo": "http://www.w3.org/2003/01/geo/wgs84_pos#",
-                         "lat": "geo:lat",
-                         "lng": "geo:long",
-                         "inbox": "http://happy-dev.fr/owl/#inbox",
-                         "object": "http://happy-dev.fr/owl/#object",
-                         "author": "http://happy-dev.fr/owl/#author",
-                         "account": "http://happy-dev.fr/owl/#account",
-                         "jabberID": "foaf:jabberID",
-                         "picture": "foaf:depiction"}
-        }
+        owner_id = "https://distant.com/users/1/"
+
+        payload = self._get_request_json(owner=owner_id)
+        payload.update({'@id': self.circle.urlid})
 
         response = self.client.patch('/circles/{}/'.format(self.circle.pk),
                                      data=json.dumps(payload), content_type='application/ld+json')
         self.assertEqual(response.status_code, 200)
 
         circle = Circle.objects.get(pk=self.circle.pk)
-        self.assertEqual(circle.owner.urlid, 'https://distant.com/users/1/')
\ No newline at end of file
+        self.assertEqual(circle.owner.urlid, owner_id)
